# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
import enum
from ..cog import variants as cogvariants
from ..models import dashboard
from ..cog import runtime as cogruntime


BucketAggregation: typing.TypeAlias = typing.Union['DateHistogram', 'Histogram', 'Terms', 'Filters', 'GeoHashGrid', 'Nested']


class DateHistogram:
    field: typing.Optional[str]
    id_val: str
    type_val: str
    settings: typing.Optional['ElasticsearchDateHistogramSettings']

    def __init__(self, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional['ElasticsearchDateHistogramSettings'] = None):
        self.field = field
        self.id_val = id_val
        self.type_val = BucketAggregationType.DATE_HISTOGRAM
        self.settings = settings

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "id": self.id_val,
            "type": self.type_val,
        }
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchDateHistogramSettings.from_json(data["settings"])        

        return cls(**args)


class BucketAggregationType(enum.StrEnum):
    TERMS = "terms"
    FILTERS = "filters"
    GEOHASH_GRID = "geohash_grid"
    DATE_HISTOGRAM = "date_histogram"
    HISTOGRAM = "histogram"
    NESTED = "nested"


class Histogram:
    field: typing.Optional[str]
    id_val: str
    type_val: str
    settings: typing.Optional['ElasticsearchHistogramSettings']

    def __init__(self, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional['ElasticsearchHistogramSettings'] = None):
        self.field = field
        self.id_val = id_val
        self.type_val = BucketAggregationType.HISTOGRAM
        self.settings = settings

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "id": self.id_val,
            "type": self.type_val,
        }
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchHistogramSettings.from_json(data["settings"])        

        return cls(**args)


class Terms:
    field: typing.Optional[str]
    id_val: str
    type_val: str
    settings: typing.Optional['ElasticsearchTermsSettings']

    def __init__(self, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional['ElasticsearchTermsSettings'] = None):
        self.field = field
        self.id_val = id_val
        self.type_val = BucketAggregationType.TERMS
        self.settings = settings

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "id": self.id_val,
            "type": self.type_val,
        }
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchTermsSettings.from_json(data["settings"])        

        return cls(**args)


class TermsOrder(enum.StrEnum):
    DESC = "desc"
    ASC = "asc"


class Filters:
    id_val: str
    type_val: str
    settings: typing.Optional['ElasticsearchFiltersSettings']

    def __init__(self, id_val: str = "", settings: typing.Optional['ElasticsearchFiltersSettings'] = None):
        self.id_val = id_val
        self.type_val = BucketAggregationType.FILTERS
        self.settings = settings

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "id": self.id_val,
            "type": self.type_val,
        }
        if self.settings is not None:
            payload["settings"] = self.settings
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchFiltersSettings.from_json(data["settings"])        

        return cls(**args)


class Filter:
    query: str
    label: str

    def __init__(self, query: str = "", label: str = ""):
        self.query = query
        self.label = label

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "query": self.query,
            "label": self.label,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "query" in data:
            args["query"] = data["query"]
        if "label" in data:
            args["label"] = data["label"]        

        return cls(**args)


class GeoHashGrid:
    field: typing.Optional[str]
    id_val: str
    type_val: str
    settings: typing.Optional['ElasticsearchGeoHashGridSettings']

    def __init__(self, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional['ElasticsearchGeoHashGridSettings'] = None):
        self.field = field
        self.id_val = id_val
        self.type_val = BucketAggregationType.GEOHASH_GRID
        self.settings = settings

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "id": self.id_val,
            "type": self.type_val,
        }
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchGeoHashGridSettings.from_json(data["settings"])        

        return cls(**args)


class Nested:
    field: typing.Optional[str]
    id_val: str
    type_val: str
    settings: typing.Optional[object]

    def __init__(self, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional[object] = None):
        self.field = field
        self.id_val = id_val
        self.type_val = BucketAggregationType.NESTED
        self.settings = settings

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "id": self.id_val,
            "type": self.type_val,
        }
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = data["settings"]        

        return cls(**args)


MetricAggregation: typing.TypeAlias = typing.Union['Count', 'MovingAverage', 'Derivative', 'CumulativeSum', 'BucketScript', 'SerialDiff', 'RawData', 'RawDocument', 'UniqueCount', 'Percentiles', 'ExtendedStats', 'Min', 'Max', 'Sum', 'Average', 'MovingFunction', 'Logs', 'Rate', 'TopMetrics']


class Count:
    type_val: str
    id_val: str
    hide: typing.Optional[bool]

    def __init__(self, id_val: str = "", hide: typing.Optional[bool] = None):
        self.type_val = MetricAggregationType.COUNT
        self.id_val = id_val
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "id" in data:
            args["id_val"] = data["id"]
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class MetricAggregationType(enum.StrEnum):
    COUNT = "count"
    AVG = "avg"
    SUM = "sum"
    MIN = "min"
    MAX = "max"
    EXTENDED_STATS = "extended_stats"
    PERCENTILES = "percentiles"
    CARDINALITY = "cardinality"
    RAW_DOCUMENT = "raw_document"
    RAW_DATA = "raw_data"
    LOGS = "logs"
    RATE = "rate"
    TOP_METRICS = "top_metrics"
    MOVING_AVG = "moving_avg"
    MOVING_FN = "moving_fn"
    DERIVATIVE = "derivative"
    SERIAL_DIFF = "serial_diff"
    CUMULATIVE_SUM = "cumulative_sum"
    BUCKET_SCRIPT = "bucket_script"


class PipelineMetricAggregationType(enum.StrEnum):
    MOVING_AVG = "moving_avg"
    MOVING_FN = "moving_fn"
    DERIVATIVE = "derivative"
    SERIAL_DIFF = "serial_diff"
    CUMULATIVE_SUM = "cumulative_sum"
    BUCKET_SCRIPT = "bucket_script"


PipelineMetricAggregation: typing.TypeAlias = typing.Union['MovingAverage', 'Derivative', 'CumulativeSum', 'BucketScript']


class MovingAverage:
    """
    #MovingAverage's settings are overridden in types.ts
    """

    pipeline_agg: typing.Optional[str]
    field: typing.Optional[str]
    type_val: str
    id_val: str
    settings: typing.Optional[dict[str, object]]
    hide: typing.Optional[bool]

    def __init__(self, pipeline_agg: typing.Optional[str] = None, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional[dict[str, object]] = None, hide: typing.Optional[bool] = None):
        self.pipeline_agg = pipeline_agg
        self.field = field
        self.type_val = MetricAggregationType.MOVING_AVG
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.pipeline_agg is not None:
            payload["pipelineAgg"] = self.pipeline_agg
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "pipelineAgg" in data:
            args["pipeline_agg"] = data["pipelineAgg"]
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = data["settings"]
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class Derivative:
    pipeline_agg: typing.Optional[str]
    field: typing.Optional[str]
    type_val: str
    id_val: str
    settings: typing.Optional['ElasticsearchDerivativeSettings']
    hide: typing.Optional[bool]

    def __init__(self, pipeline_agg: typing.Optional[str] = None, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional['ElasticsearchDerivativeSettings'] = None, hide: typing.Optional[bool] = None):
        self.pipeline_agg = pipeline_agg
        self.field = field
        self.type_val = MetricAggregationType.DERIVATIVE
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.pipeline_agg is not None:
            payload["pipelineAgg"] = self.pipeline_agg
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "pipelineAgg" in data:
            args["pipeline_agg"] = data["pipelineAgg"]
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchDerivativeSettings.from_json(data["settings"])
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class CumulativeSum:
    pipeline_agg: typing.Optional[str]
    field: typing.Optional[str]
    type_val: str
    id_val: str
    settings: typing.Optional['ElasticsearchCumulativeSumSettings']
    hide: typing.Optional[bool]

    def __init__(self, pipeline_agg: typing.Optional[str] = None, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional['ElasticsearchCumulativeSumSettings'] = None, hide: typing.Optional[bool] = None):
        self.pipeline_agg = pipeline_agg
        self.field = field
        self.type_val = MetricAggregationType.CUMULATIVE_SUM
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.pipeline_agg is not None:
            payload["pipelineAgg"] = self.pipeline_agg
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "pipelineAgg" in data:
            args["pipeline_agg"] = data["pipelineAgg"]
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchCumulativeSumSettings.from_json(data["settings"])
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class BucketScript:
    type_val: str
    pipeline_variables: typing.Optional[list['PipelineVariable']]
    id_val: str
    settings: typing.Optional['ElasticsearchBucketScriptSettings']
    hide: typing.Optional[bool]

    def __init__(self, pipeline_variables: typing.Optional[list['PipelineVariable']] = None, id_val: str = "", settings: typing.Optional['ElasticsearchBucketScriptSettings'] = None, hide: typing.Optional[bool] = None):
        self.type_val = MetricAggregationType.BUCKET_SCRIPT
        self.pipeline_variables = pipeline_variables
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.pipeline_variables is not None:
            payload["pipelineVariables"] = self.pipeline_variables
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "pipelineVariables" in data:
            args["pipeline_variables"] = [PipelineVariable.from_json(item) for item in data["pipelineVariables"]]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchBucketScriptSettings.from_json(data["settings"])
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class PipelineVariable:
    name: str
    pipeline_agg: str

    def __init__(self, name: str = "", pipeline_agg: str = ""):
        self.name = name
        self.pipeline_agg = pipeline_agg

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "name": self.name,
            "pipelineAgg": self.pipeline_agg,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "name" in data:
            args["name"] = data["name"]
        if "pipelineAgg" in data:
            args["pipeline_agg"] = data["pipelineAgg"]        

        return cls(**args)


InlineScript: typing.TypeAlias = typing.Union[str, 'ElasticsearchInlineScript']


MetricAggregationWithSettings: typing.TypeAlias = typing.Union['BucketScript', 'CumulativeSum', 'Derivative', 'SerialDiff', 'RawData', 'RawDocument', 'UniqueCount', 'Percentiles', 'ExtendedStats', 'Min', 'Max', 'Sum', 'Average', 'MovingAverage', 'MovingFunction', 'Logs', 'Rate', 'TopMetrics']


class SerialDiff:
    pipeline_agg: typing.Optional[str]
    field: typing.Optional[str]
    type_val: str
    id_val: str
    settings: typing.Optional['ElasticsearchSerialDiffSettings']
    hide: typing.Optional[bool]

    def __init__(self, pipeline_agg: typing.Optional[str] = None, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional['ElasticsearchSerialDiffSettings'] = None, hide: typing.Optional[bool] = None):
        self.pipeline_agg = pipeline_agg
        self.field = field
        self.type_val = MetricAggregationType.SERIAL_DIFF
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.pipeline_agg is not None:
            payload["pipelineAgg"] = self.pipeline_agg
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "pipelineAgg" in data:
            args["pipeline_agg"] = data["pipelineAgg"]
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchSerialDiffSettings.from_json(data["settings"])
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class RawData:
    type_val: str
    id_val: str
    settings: typing.Optional['ElasticsearchRawDataSettings']
    hide: typing.Optional[bool]

    def __init__(self, id_val: str = "", settings: typing.Optional['ElasticsearchRawDataSettings'] = None, hide: typing.Optional[bool] = None):
        self.type_val = MetricAggregationType.RAW_DATA
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchRawDataSettings.from_json(data["settings"])
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class RawDocument:
    type_val: str
    id_val: str
    settings: typing.Optional['ElasticsearchRawDocumentSettings']
    hide: typing.Optional[bool]

    def __init__(self, id_val: str = "", settings: typing.Optional['ElasticsearchRawDocumentSettings'] = None, hide: typing.Optional[bool] = None):
        self.type_val = MetricAggregationType.RAW_DOCUMENT
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchRawDocumentSettings.from_json(data["settings"])
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class UniqueCount:
    type_val: str
    field: typing.Optional[str]
    id_val: str
    settings: typing.Optional['ElasticsearchUniqueCountSettings']
    hide: typing.Optional[bool]

    def __init__(self, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional['ElasticsearchUniqueCountSettings'] = None, hide: typing.Optional[bool] = None):
        self.type_val = MetricAggregationType.CARDINALITY
        self.field = field
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchUniqueCountSettings.from_json(data["settings"])
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class Percentiles:
    type_val: str
    field: typing.Optional[str]
    id_val: str
    settings: typing.Optional['ElasticsearchPercentilesSettings']
    hide: typing.Optional[bool]

    def __init__(self, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional['ElasticsearchPercentilesSettings'] = None, hide: typing.Optional[bool] = None):
        self.type_val = MetricAggregationType.PERCENTILES
        self.field = field
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchPercentilesSettings.from_json(data["settings"])
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class ExtendedStats:
    type_val: str
    settings: typing.Optional['ElasticsearchExtendedStatsSettings']
    field: typing.Optional[str]
    id_val: str
    meta: typing.Optional[object]
    hide: typing.Optional[bool]

    def __init__(self, settings: typing.Optional['ElasticsearchExtendedStatsSettings'] = None, field: typing.Optional[str] = None, id_val: str = "", meta: typing.Optional[object] = None, hide: typing.Optional[bool] = None):
        self.type_val = MetricAggregationType.EXTENDED_STATS
        self.settings = settings
        self.field = field
        self.id_val = id_val
        self.meta = meta
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.field is not None:
            payload["field"] = self.field
        if self.meta is not None:
            payload["meta"] = self.meta
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "settings" in data:
            args["settings"] = ElasticsearchExtendedStatsSettings.from_json(data["settings"])
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "meta" in data:
            args["meta"] = data["meta"]
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class Min:
    type_val: str
    field: typing.Optional[str]
    id_val: str
    settings: typing.Optional['ElasticsearchMinSettings']
    hide: typing.Optional[bool]

    def __init__(self, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional['ElasticsearchMinSettings'] = None, hide: typing.Optional[bool] = None):
        self.type_val = MetricAggregationType.MIN
        self.field = field
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchMinSettings.from_json(data["settings"])
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class Max:
    type_val: str
    field: typing.Optional[str]
    id_val: str
    settings: typing.Optional['ElasticsearchMaxSettings']
    hide: typing.Optional[bool]

    def __init__(self, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional['ElasticsearchMaxSettings'] = None, hide: typing.Optional[bool] = None):
        self.type_val = MetricAggregationType.MAX
        self.field = field
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchMaxSettings.from_json(data["settings"])
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class Sum:
    type_val: str
    field: typing.Optional[str]
    id_val: str
    settings: typing.Optional['ElasticsearchSumSettings']
    hide: typing.Optional[bool]

    def __init__(self, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional['ElasticsearchSumSettings'] = None, hide: typing.Optional[bool] = None):
        self.type_val = MetricAggregationType.SUM
        self.field = field
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchSumSettings.from_json(data["settings"])
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class Average:
    type_val: str
    field: typing.Optional[str]
    id_val: str
    settings: typing.Optional['ElasticsearchAverageSettings']
    hide: typing.Optional[bool]

    def __init__(self, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional['ElasticsearchAverageSettings'] = None, hide: typing.Optional[bool] = None):
        self.type_val = MetricAggregationType.AVG
        self.field = field
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchAverageSettings.from_json(data["settings"])
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class MovingFunction:
    pipeline_agg: typing.Optional[str]
    field: typing.Optional[str]
    type_val: str
    id_val: str
    settings: typing.Optional['ElasticsearchMovingFunctionSettings']
    hide: typing.Optional[bool]

    def __init__(self, pipeline_agg: typing.Optional[str] = None, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional['ElasticsearchMovingFunctionSettings'] = None, hide: typing.Optional[bool] = None):
        self.pipeline_agg = pipeline_agg
        self.field = field
        self.type_val = MetricAggregationType.MOVING_FN
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.pipeline_agg is not None:
            payload["pipelineAgg"] = self.pipeline_agg
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "pipelineAgg" in data:
            args["pipeline_agg"] = data["pipelineAgg"]
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchMovingFunctionSettings.from_json(data["settings"])
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class Logs:
    type_val: str
    id_val: str
    settings: typing.Optional['ElasticsearchLogsSettings']
    hide: typing.Optional[bool]

    def __init__(self, id_val: str = "", settings: typing.Optional['ElasticsearchLogsSettings'] = None, hide: typing.Optional[bool] = None):
        self.type_val = MetricAggregationType.LOGS
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchLogsSettings.from_json(data["settings"])
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class Rate:
    type_val: str
    field: typing.Optional[str]
    id_val: str
    settings: typing.Optional['ElasticsearchRateSettings']
    hide: typing.Optional[bool]

    def __init__(self, field: typing.Optional[str] = None, id_val: str = "", settings: typing.Optional['ElasticsearchRateSettings'] = None, hide: typing.Optional[bool] = None):
        self.type_val = MetricAggregationType.RATE
        self.field = field
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchRateSettings.from_json(data["settings"])
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class TopMetrics:
    type_val: str
    id_val: str
    settings: typing.Optional['ElasticsearchTopMetricsSettings']
    hide: typing.Optional[bool]

    def __init__(self, id_val: str = "", settings: typing.Optional['ElasticsearchTopMetricsSettings'] = None, hide: typing.Optional[bool] = None):
        self.type_val = MetricAggregationType.TOP_METRICS
        self.id_val = id_val
        self.settings = settings
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "id" in data:
            args["id_val"] = data["id"]
        if "settings" in data:
            args["settings"] = ElasticsearchTopMetricsSettings.from_json(data["settings"])
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class BaseBucketAggregation:
    id_val: str
    type_val: 'BucketAggregationType'
    settings: typing.Optional[object]

    def __init__(self, id_val: str = "", type_val: typing.Optional['BucketAggregationType'] = None, settings: typing.Optional[object] = None):
        self.id_val = id_val
        self.type_val = type_val if type_val is not None else BucketAggregationType.TERMS
        self.settings = settings

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "id": self.id_val,
            "type": self.type_val,
        }
        if self.settings is not None:
            payload["settings"] = self.settings
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "id" in data:
            args["id_val"] = data["id"]
        if "type" in data:
            args["type_val"] = data["type"]
        if "settings" in data:
            args["settings"] = data["settings"]        

        return cls(**args)


class BucketAggregationWithField:
    field: typing.Optional[str]
    id_val: str
    type_val: 'BucketAggregationType'
    settings: typing.Optional[object]

    def __init__(self, field: typing.Optional[str] = None, id_val: str = "", type_val: typing.Optional['BucketAggregationType'] = None, settings: typing.Optional[object] = None):
        self.field = field
        self.id_val = id_val
        self.type_val = type_val if type_val is not None else BucketAggregationType.TERMS
        self.settings = settings

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "id": self.id_val,
            "type": self.type_val,
        }
        if self.field is not None:
            payload["field"] = self.field
        if self.settings is not None:
            payload["settings"] = self.settings
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "field" in data:
            args["field"] = data["field"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "type" in data:
            args["type_val"] = data["type"]
        if "settings" in data:
            args["settings"] = data["settings"]        

        return cls(**args)


class DateHistogramSettings:
    interval: typing.Optional[str]
    min_doc_count: typing.Optional[str]
    trim_edges: typing.Optional[str]
    offset: typing.Optional[str]
    time_zone: typing.Optional[str]

    def __init__(self, interval: typing.Optional[str] = None, min_doc_count: typing.Optional[str] = None, trim_edges: typing.Optional[str] = None, offset: typing.Optional[str] = None, time_zone: typing.Optional[str] = None):
        self.interval = interval
        self.min_doc_count = min_doc_count
        self.trim_edges = trim_edges
        self.offset = offset
        self.time_zone = time_zone

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.interval is not None:
            payload["interval"] = self.interval
        if self.min_doc_count is not None:
            payload["min_doc_count"] = self.min_doc_count
        if self.trim_edges is not None:
            payload["trimEdges"] = self.trim_edges
        if self.offset is not None:
            payload["offset"] = self.offset
        if self.time_zone is not None:
            payload["timeZone"] = self.time_zone
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "interval" in data:
            args["interval"] = data["interval"]
        if "min_doc_count" in data:
            args["min_doc_count"] = data["min_doc_count"]
        if "trimEdges" in data:
            args["trim_edges"] = data["trimEdges"]
        if "offset" in data:
            args["offset"] = data["offset"]
        if "timeZone" in data:
            args["time_zone"] = data["timeZone"]        

        return cls(**args)


class HistogramSettings:
    interval: typing.Optional[str]
    min_doc_count: typing.Optional[str]

    def __init__(self, interval: typing.Optional[str] = None, min_doc_count: typing.Optional[str] = None):
        self.interval = interval
        self.min_doc_count = min_doc_count

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.interval is not None:
            payload["interval"] = self.interval
        if self.min_doc_count is not None:
            payload["min_doc_count"] = self.min_doc_count
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "interval" in data:
            args["interval"] = data["interval"]
        if "min_doc_count" in data:
            args["min_doc_count"] = data["min_doc_count"]        

        return cls(**args)


class TermsSettings:
    order: typing.Optional['TermsOrder']
    size: typing.Optional[str]
    min_doc_count: typing.Optional[str]
    order_by: typing.Optional[str]
    missing: typing.Optional[str]

    def __init__(self, order: typing.Optional['TermsOrder'] = None, size: typing.Optional[str] = None, min_doc_count: typing.Optional[str] = None, order_by: typing.Optional[str] = None, missing: typing.Optional[str] = None):
        self.order = order
        self.size = size
        self.min_doc_count = min_doc_count
        self.order_by = order_by
        self.missing = missing

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.order is not None:
            payload["order"] = self.order
        if self.size is not None:
            payload["size"] = self.size
        if self.min_doc_count is not None:
            payload["min_doc_count"] = self.min_doc_count
        if self.order_by is not None:
            payload["orderBy"] = self.order_by
        if self.missing is not None:
            payload["missing"] = self.missing
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "order" in data:
            args["order"] = data["order"]
        if "size" in data:
            args["size"] = data["size"]
        if "min_doc_count" in data:
            args["min_doc_count"] = data["min_doc_count"]
        if "orderBy" in data:
            args["order_by"] = data["orderBy"]
        if "missing" in data:
            args["missing"] = data["missing"]        

        return cls(**args)


class FiltersSettings:
    filters: typing.Optional[list['Filter']]

    def __init__(self, filters: typing.Optional[list['Filter']] = None):
        self.filters = filters

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.filters is not None:
            payload["filters"] = self.filters
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "filters" in data:
            args["filters"] = [Filter.from_json(item) for item in data["filters"]]        

        return cls(**args)


class GeoHashGridSettings:
    precision: typing.Optional[str]

    def __init__(self, precision: typing.Optional[str] = None):
        self.precision = precision

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.precision is not None:
            payload["precision"] = self.precision
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "precision" in data:
            args["precision"] = data["precision"]        

        return cls(**args)


class BaseMetricAggregation:
    type_val: 'MetricAggregationType'
    id_val: str
    hide: typing.Optional[bool]

    def __init__(self, type_val: typing.Optional['MetricAggregationType'] = None, id_val: str = "", hide: typing.Optional[bool] = None):
        self.type_val = type_val if type_val is not None else MetricAggregationType.COUNT
        self.id_val = id_val
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "type" in data:
            args["type_val"] = data["type"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class MetricAggregationWithField:
    field: typing.Optional[str]
    type_val: 'MetricAggregationType'
    id_val: str
    hide: typing.Optional[bool]

    def __init__(self, field: typing.Optional[str] = None, type_val: typing.Optional['MetricAggregationType'] = None, id_val: str = "", hide: typing.Optional[bool] = None):
        self.field = field
        self.type_val = type_val if type_val is not None else MetricAggregationType.COUNT
        self.id_val = id_val
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.field is not None:
            payload["field"] = self.field
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "field" in data:
            args["field"] = data["field"]
        if "type" in data:
            args["type_val"] = data["type"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class MetricAggregationWithMissingSupport:
    settings: typing.Optional['ElasticsearchMetricAggregationWithMissingSupportSettings']
    type_val: 'MetricAggregationType'
    id_val: str
    hide: typing.Optional[bool]

    def __init__(self, settings: typing.Optional['ElasticsearchMetricAggregationWithMissingSupportSettings'] = None, type_val: typing.Optional['MetricAggregationType'] = None, id_val: str = "", hide: typing.Optional[bool] = None):
        self.settings = settings
        self.type_val = type_val if type_val is not None else MetricAggregationType.COUNT
        self.id_val = id_val
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "settings" in data:
            args["settings"] = ElasticsearchMetricAggregationWithMissingSupportSettings.from_json(data["settings"])
        if "type" in data:
            args["type_val"] = data["type"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class MetricAggregationWithInlineScript:
    settings: typing.Optional['ElasticsearchMetricAggregationWithInlineScriptSettings']
    type_val: 'MetricAggregationType'
    id_val: str
    hide: typing.Optional[bool]

    def __init__(self, settings: typing.Optional['ElasticsearchMetricAggregationWithInlineScriptSettings'] = None, type_val: typing.Optional['MetricAggregationType'] = None, id_val: str = "", hide: typing.Optional[bool] = None):
        self.settings = settings
        self.type_val = type_val if type_val is not None else MetricAggregationType.COUNT
        self.id_val = id_val
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.settings is not None:
            payload["settings"] = self.settings
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "settings" in data:
            args["settings"] = ElasticsearchMetricAggregationWithInlineScriptSettings.from_json(data["settings"])
        if "type" in data:
            args["type_val"] = data["type"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class ExtendedStatMetaType(enum.StrEnum):
    AVG = "avg"
    MIN = "min"
    MAX = "max"
    SUM = "sum"
    COUNT = "count"
    STD_DEVIATION = "std_deviation"
    STD_DEVIATION_BOUNDS_UPPER = "std_deviation_bounds_upper"
    STD_DEVIATION_BOUNDS_LOWER = "std_deviation_bounds_lower"


class ExtendedStat:
    label: str
    value: 'ExtendedStatMetaType'

    def __init__(self, label: str = "", value: typing.Optional['ExtendedStatMetaType'] = None):
        self.label = label
        self.value = value if value is not None else ExtendedStatMetaType.AVG

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "label": self.label,
            "value": self.value,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "label" in data:
            args["label"] = data["label"]
        if "value" in data:
            args["value"] = data["value"]        

        return cls(**args)


class BasePipelineMetricAggregation:
    pipeline_agg: typing.Optional[str]
    field: typing.Optional[str]
    type_val: str
    id_val: str
    hide: typing.Optional[bool]

    def __init__(self, pipeline_agg: typing.Optional[str] = None, field: typing.Optional[str] = None, type_val: str = "", id_val: str = "", hide: typing.Optional[bool] = None):
        self.pipeline_agg = pipeline_agg
        self.field = field
        self.type_val = type_val
        self.id_val = id_val
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.pipeline_agg is not None:
            payload["pipelineAgg"] = self.pipeline_agg
        if self.field is not None:
            payload["field"] = self.field
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "pipelineAgg" in data:
            args["pipeline_agg"] = data["pipelineAgg"]
        if "field" in data:
            args["field"] = data["field"]
        if "type" in data:
            args["type_val"] = data["type"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class PipelineMetricAggregationWithMultipleBucketPaths:
    pipeline_variables: typing.Optional[list['PipelineVariable']]
    type_val: 'MetricAggregationType'
    id_val: str
    hide: typing.Optional[bool]

    def __init__(self, pipeline_variables: typing.Optional[list['PipelineVariable']] = None, type_val: typing.Optional['MetricAggregationType'] = None, id_val: str = "", hide: typing.Optional[bool] = None):
        self.pipeline_variables = pipeline_variables
        self.type_val = type_val if type_val is not None else MetricAggregationType.COUNT
        self.id_val = id_val
        self.hide = hide

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
            "id": self.id_val,
        }
        if self.pipeline_variables is not None:
            payload["pipelineVariables"] = self.pipeline_variables
        if self.hide is not None:
            payload["hide"] = self.hide
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "pipelineVariables" in data:
            args["pipeline_variables"] = [PipelineVariable.from_json(item) for item in data["pipelineVariables"]]
        if "type" in data:
            args["type_val"] = data["type"]
        if "id" in data:
            args["id_val"] = data["id"]
        if "hide" in data:
            args["hide"] = data["hide"]        

        return cls(**args)


class MovingAverageModel(enum.StrEnum):
    SIMPLE = "simple"
    LINEAR = "linear"
    EWMA = "ewma"
    HOLT = "holt"
    HOLT_WINTERS = "holt_winters"


class MovingAverageModelOption:
    label: str
    value: 'MovingAverageModel'

    def __init__(self, label: str = "", value: typing.Optional['MovingAverageModel'] = None):
        self.label = label
        self.value = value if value is not None else MovingAverageModel.SIMPLE

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "label": self.label,
            "value": self.value,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "label" in data:
            args["label"] = data["label"]
        if "value" in data:
            args["value"] = data["value"]        

        return cls(**args)


class BaseMovingAverageModelSettings:
    model: 'MovingAverageModel'
    window: str
    predict: str

    def __init__(self, model: typing.Optional['MovingAverageModel'] = None, window: str = "", predict: str = ""):
        self.model = model if model is not None else MovingAverageModel.SIMPLE
        self.window = window
        self.predict = predict

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "model": self.model,
            "window": self.window,
            "predict": self.predict,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "model" in data:
            args["model"] = data["model"]
        if "window" in data:
            args["window"] = data["window"]
        if "predict" in data:
            args["predict"] = data["predict"]        

        return cls(**args)


class MovingAverageSimpleModelSettings:
    model: str
    window: str
    predict: str

    def __init__(self, window: str = "", predict: str = ""):
        self.model = MovingAverageModel.SIMPLE
        self.window = window
        self.predict = predict

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "model": self.model,
            "window": self.window,
            "predict": self.predict,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "window" in data:
            args["window"] = data["window"]
        if "predict" in data:
            args["predict"] = data["predict"]        

        return cls(**args)


class MovingAverageLinearModelSettings:
    model: str
    window: str
    predict: str

    def __init__(self, window: str = "", predict: str = ""):
        self.model = MovingAverageModel.LINEAR
        self.window = window
        self.predict = predict

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "model": self.model,
            "window": self.window,
            "predict": self.predict,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "window" in data:
            args["window"] = data["window"]
        if "predict" in data:
            args["predict"] = data["predict"]        

        return cls(**args)


class MovingAverageEWMAModelSettings:
    model: str
    settings: typing.Optional['ElasticsearchMovingAverageEWMAModelSettingsSettings']
    window: str
    minimize: bool
    predict: str

    def __init__(self, settings: typing.Optional['ElasticsearchMovingAverageEWMAModelSettingsSettings'] = None, window: str = "", minimize: bool = False, predict: str = ""):
        self.model = MovingAverageModel.EWMA
        self.settings = settings
        self.window = window
        self.minimize = minimize
        self.predict = predict

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "model": self.model,
            "window": self.window,
            "minimize": self.minimize,
            "predict": self.predict,
        }
        if self.settings is not None:
            payload["settings"] = self.settings
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "settings" in data:
            args["settings"] = ElasticsearchMovingAverageEWMAModelSettingsSettings.from_json(data["settings"])
        if "window" in data:
            args["window"] = data["window"]
        if "minimize" in data:
            args["minimize"] = data["minimize"]
        if "predict" in data:
            args["predict"] = data["predict"]        

        return cls(**args)


class MovingAverageHoltModelSettings:
    model: str
    settings: 'ElasticsearchMovingAverageHoltModelSettingsSettings'
    window: str
    minimize: bool
    predict: str

    def __init__(self, settings: typing.Optional['ElasticsearchMovingAverageHoltModelSettingsSettings'] = None, window: str = "", minimize: bool = False, predict: str = ""):
        self.model = MovingAverageModel.HOLT
        self.settings = settings if settings is not None else ElasticsearchMovingAverageHoltModelSettingsSettings()
        self.window = window
        self.minimize = minimize
        self.predict = predict

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "model": self.model,
            "settings": self.settings,
            "window": self.window,
            "minimize": self.minimize,
            "predict": self.predict,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "settings" in data:
            args["settings"] = ElasticsearchMovingAverageHoltModelSettingsSettings.from_json(data["settings"])
        if "window" in data:
            args["window"] = data["window"]
        if "minimize" in data:
            args["minimize"] = data["minimize"]
        if "predict" in data:
            args["predict"] = data["predict"]        

        return cls(**args)


class MovingAverageHoltWintersModelSettings:
    model: str
    settings: 'ElasticsearchMovingAverageHoltWintersModelSettingsSettings'
    window: str
    minimize: bool
    predict: str

    def __init__(self, settings: typing.Optional['ElasticsearchMovingAverageHoltWintersModelSettingsSettings'] = None, window: str = "", minimize: bool = False, predict: str = ""):
        self.model = MovingAverageModel.HOLT_WINTERS
        self.settings = settings if settings is not None else ElasticsearchMovingAverageHoltWintersModelSettingsSettings()
        self.window = window
        self.minimize = minimize
        self.predict = predict

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "model": self.model,
            "settings": self.settings,
            "window": self.window,
            "minimize": self.minimize,
            "predict": self.predict,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "settings" in data:
            args["settings"] = ElasticsearchMovingAverageHoltWintersModelSettingsSettings.from_json(data["settings"])
        if "window" in data:
            args["window"] = data["window"]
        if "minimize" in data:
            args["minimize"] = data["minimize"]
        if "predict" in data:
            args["predict"] = data["predict"]        

        return cls(**args)


class Dataquery(cogvariants.Dataquery):
    # Alias pattern
    alias: typing.Optional[str]
    # Lucene query
    query: typing.Optional[str]
    # Name of time field
    time_field: typing.Optional[str]
    # List of bucket aggregations
    bucket_aggs: typing.Optional[list['BucketAggregation']]
    # List of metric aggregations
    metrics: typing.Optional[list['MetricAggregation']]
    # A unique identifier for the query within the list of targets.
    # In server side expressions, the refId is used as a variable name to identify results.
    # By default, the UI will assign A->Z; however setting meaningful names may be useful.
    ref_id: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # Note this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Specify the query flavor
    # TODO make this required and give it a default
    query_type: typing.Optional[str]
    # For mixed data sources the selected datasource is on the query level.
    # For non mixed scenarios this is undefined.
    # TODO find a better way to do this ^ that's friendly to schema
    # TODO this shouldn't be unknown but DataSourceRef | null
    datasource: typing.Optional[dashboard.DataSourceRef]

    def __init__(self, alias: typing.Optional[str] = None, query: typing.Optional[str] = None, time_field: typing.Optional[str] = None, bucket_aggs: typing.Optional[list['BucketAggregation']] = None, metrics: typing.Optional[list['MetricAggregation']] = None, ref_id: str = "", hide: typing.Optional[bool] = None, query_type: typing.Optional[str] = None, datasource: typing.Optional[dashboard.DataSourceRef] = None):
        self.alias = alias
        self.query = query
        self.time_field = time_field
        self.bucket_aggs = bucket_aggs
        self.metrics = metrics
        self.ref_id = ref_id
        self.hide = hide
        self.query_type = query_type
        self.datasource = datasource

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "refId": self.ref_id,
        }
        if self.alias is not None:
            payload["alias"] = self.alias
        if self.query is not None:
            payload["query"] = self.query
        if self.time_field is not None:
            payload["timeField"] = self.time_field
        if self.bucket_aggs is not None:
            payload["bucketAggs"] = self.bucket_aggs
        if self.metrics is not None:
            payload["metrics"] = self.metrics
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "alias" in data:
            args["alias"] = data["alias"]
        if "query" in data:
            args["query"] = data["query"]
        if "timeField" in data:
            args["time_field"] = data["timeField"]
        if "bucketAggs" in data:
            decoding_map_bucketAggs_array_ref_union: dict[str, typing.Union[typing.Type[DateHistogram], typing.Type[Filters], typing.Type[GeoHashGrid], typing.Type[Histogram], typing.Type[Nested], typing.Type[Terms]]] = {"date_histogram": DateHistogram, "filters": Filters, "geohash_grid": GeoHashGrid, "histogram": Histogram, "nested": Nested, "terms": Terms}
            args["bucket_aggs"] = [decoding_map_bucketAggs_array_ref_union[item["type"]].from_json(item) for item in data["bucketAggs"]]
        if "metrics" in data:
            decoding_map_metrics_array_ref_union: dict[str, typing.Union[typing.Type[Average], typing.Type[BucketScript], typing.Type[UniqueCount], typing.Type[Count], typing.Type[CumulativeSum], typing.Type[Derivative], typing.Type[ExtendedStats], typing.Type[Logs], typing.Type[Max], typing.Type[Min], typing.Type[MovingAverage], typing.Type[MovingFunction], typing.Type[Percentiles], typing.Type[Rate], typing.Type[RawData], typing.Type[RawDocument], typing.Type[SerialDiff], typing.Type[Sum], typing.Type[TopMetrics]]] = {"avg": Average, "bucket_script": BucketScript, "cardinality": UniqueCount, "count": Count, "cumulative_sum": CumulativeSum, "derivative": Derivative, "extended_stats": ExtendedStats, "logs": Logs, "max": Max, "min": Min, "moving_avg": MovingAverage, "moving_fn": MovingFunction, "percentiles": Percentiles, "rate": Rate, "raw_data": RawData, "raw_document": RawDocument, "serial_diff": SerialDiff, "sum": Sum, "top_metrics": TopMetrics}
            args["metrics"] = [decoding_map_metrics_array_ref_union[item["type"]].from_json(item) for item in data["metrics"]]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])        

        return cls(**args)


class ElasticsearchDateHistogramSettings:
    interval: typing.Optional[str]
    min_doc_count: typing.Optional[str]
    trim_edges: typing.Optional[str]
    offset: typing.Optional[str]
    time_zone: typing.Optional[str]

    def __init__(self, interval: typing.Optional[str] = None, min_doc_count: typing.Optional[str] = None, trim_edges: typing.Optional[str] = None, offset: typing.Optional[str] = None, time_zone: typing.Optional[str] = None):
        self.interval = interval
        self.min_doc_count = min_doc_count
        self.trim_edges = trim_edges
        self.offset = offset
        self.time_zone = time_zone

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.interval is not None:
            payload["interval"] = self.interval
        if self.min_doc_count is not None:
            payload["min_doc_count"] = self.min_doc_count
        if self.trim_edges is not None:
            payload["trimEdges"] = self.trim_edges
        if self.offset is not None:
            payload["offset"] = self.offset
        if self.time_zone is not None:
            payload["timeZone"] = self.time_zone
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "interval" in data:
            args["interval"] = data["interval"]
        if "min_doc_count" in data:
            args["min_doc_count"] = data["min_doc_count"]
        if "trimEdges" in data:
            args["trim_edges"] = data["trimEdges"]
        if "offset" in data:
            args["offset"] = data["offset"]
        if "timeZone" in data:
            args["time_zone"] = data["timeZone"]        

        return cls(**args)


class ElasticsearchHistogramSettings:
    interval: typing.Optional[str]
    min_doc_count: typing.Optional[str]

    def __init__(self, interval: typing.Optional[str] = None, min_doc_count: typing.Optional[str] = None):
        self.interval = interval
        self.min_doc_count = min_doc_count

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.interval is not None:
            payload["interval"] = self.interval
        if self.min_doc_count is not None:
            payload["min_doc_count"] = self.min_doc_count
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "interval" in data:
            args["interval"] = data["interval"]
        if "min_doc_count" in data:
            args["min_doc_count"] = data["min_doc_count"]        

        return cls(**args)


class ElasticsearchTermsSettings:
    order: typing.Optional['TermsOrder']
    size: typing.Optional[str]
    min_doc_count: typing.Optional[str]
    order_by: typing.Optional[str]
    missing: typing.Optional[str]

    def __init__(self, order: typing.Optional['TermsOrder'] = None, size: typing.Optional[str] = None, min_doc_count: typing.Optional[str] = None, order_by: typing.Optional[str] = None, missing: typing.Optional[str] = None):
        self.order = order
        self.size = size
        self.min_doc_count = min_doc_count
        self.order_by = order_by
        self.missing = missing

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.order is not None:
            payload["order"] = self.order
        if self.size is not None:
            payload["size"] = self.size
        if self.min_doc_count is not None:
            payload["min_doc_count"] = self.min_doc_count
        if self.order_by is not None:
            payload["orderBy"] = self.order_by
        if self.missing is not None:
            payload["missing"] = self.missing
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "order" in data:
            args["order"] = data["order"]
        if "size" in data:
            args["size"] = data["size"]
        if "min_doc_count" in data:
            args["min_doc_count"] = data["min_doc_count"]
        if "orderBy" in data:
            args["order_by"] = data["orderBy"]
        if "missing" in data:
            args["missing"] = data["missing"]        

        return cls(**args)


class ElasticsearchFiltersSettings:
    filters: typing.Optional[list['Filter']]

    def __init__(self, filters: typing.Optional[list['Filter']] = None):
        self.filters = filters

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.filters is not None:
            payload["filters"] = self.filters
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "filters" in data:
            args["filters"] = [Filter.from_json(item) for item in data["filters"]]        

        return cls(**args)


class ElasticsearchGeoHashGridSettings:
    precision: typing.Optional[str]

    def __init__(self, precision: typing.Optional[str] = None):
        self.precision = precision

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.precision is not None:
            payload["precision"] = self.precision
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "precision" in data:
            args["precision"] = data["precision"]        

        return cls(**args)


class ElasticsearchDerivativeSettings:
    unit: typing.Optional[str]

    def __init__(self, unit: typing.Optional[str] = None):
        self.unit = unit

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.unit is not None:
            payload["unit"] = self.unit
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "unit" in data:
            args["unit"] = data["unit"]        

        return cls(**args)


class ElasticsearchCumulativeSumSettings:
    format_val: typing.Optional[str]

    def __init__(self, format_val: typing.Optional[str] = None):
        self.format_val = format_val

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.format_val is not None:
            payload["format"] = self.format_val
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "format" in data:
            args["format_val"] = data["format"]        

        return cls(**args)


class ElasticsearchBucketScriptSettings:
    script: typing.Optional['InlineScript']

    def __init__(self, script: typing.Optional['InlineScript'] = None):
        self.script = script

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.script is not None:
            payload["script"] = self.script
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "script" in data:
            args["script"] = data["script"]        

        return cls(**args)


class ElasticsearchInlineScript:
    inline: typing.Optional[str]

    def __init__(self, inline: typing.Optional[str] = None):
        self.inline = inline

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.inline is not None:
            payload["inline"] = self.inline
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "inline" in data:
            args["inline"] = data["inline"]        

        return cls(**args)


class ElasticsearchSerialDiffSettings:
    lag: typing.Optional[str]

    def __init__(self, lag: typing.Optional[str] = None):
        self.lag = lag

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.lag is not None:
            payload["lag"] = self.lag
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "lag" in data:
            args["lag"] = data["lag"]        

        return cls(**args)


class ElasticsearchRawDataSettings:
    size: typing.Optional[str]

    def __init__(self, size: typing.Optional[str] = None):
        self.size = size

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.size is not None:
            payload["size"] = self.size
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "size" in data:
            args["size"] = data["size"]        

        return cls(**args)


class ElasticsearchRawDocumentSettings:
    size: typing.Optional[str]

    def __init__(self, size: typing.Optional[str] = None):
        self.size = size

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.size is not None:
            payload["size"] = self.size
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "size" in data:
            args["size"] = data["size"]        

        return cls(**args)


class ElasticsearchUniqueCountSettings:
    precision_threshold: typing.Optional[str]
    missing: typing.Optional[str]

    def __init__(self, precision_threshold: typing.Optional[str] = None, missing: typing.Optional[str] = None):
        self.precision_threshold = precision_threshold
        self.missing = missing

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.precision_threshold is not None:
            payload["precision_threshold"] = self.precision_threshold
        if self.missing is not None:
            payload["missing"] = self.missing
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "precision_threshold" in data:
            args["precision_threshold"] = data["precision_threshold"]
        if "missing" in data:
            args["missing"] = data["missing"]        

        return cls(**args)


class ElasticsearchPercentilesSettings:
    script: typing.Optional['InlineScript']
    missing: typing.Optional[str]
    percents: typing.Optional[list[str]]

    def __init__(self, script: typing.Optional['InlineScript'] = None, missing: typing.Optional[str] = None, percents: typing.Optional[list[str]] = None):
        self.script = script
        self.missing = missing
        self.percents = percents

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.script is not None:
            payload["script"] = self.script
        if self.missing is not None:
            payload["missing"] = self.missing
        if self.percents is not None:
            payload["percents"] = self.percents
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "script" in data:
            args["script"] = data["script"]
        if "missing" in data:
            args["missing"] = data["missing"]
        if "percents" in data:
            args["percents"] = data["percents"]        

        return cls(**args)


class ElasticsearchExtendedStatsSettings:
    script: typing.Optional['InlineScript']
    missing: typing.Optional[str]
    sigma: typing.Optional[str]

    def __init__(self, script: typing.Optional['InlineScript'] = None, missing: typing.Optional[str] = None, sigma: typing.Optional[str] = None):
        self.script = script
        self.missing = missing
        self.sigma = sigma

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.script is not None:
            payload["script"] = self.script
        if self.missing is not None:
            payload["missing"] = self.missing
        if self.sigma is not None:
            payload["sigma"] = self.sigma
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "script" in data:
            args["script"] = data["script"]
        if "missing" in data:
            args["missing"] = data["missing"]
        if "sigma" in data:
            args["sigma"] = data["sigma"]        

        return cls(**args)


class ElasticsearchMinSettings:
    script: typing.Optional['InlineScript']
    missing: typing.Optional[str]

    def __init__(self, script: typing.Optional['InlineScript'] = None, missing: typing.Optional[str] = None):
        self.script = script
        self.missing = missing

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.script is not None:
            payload["script"] = self.script
        if self.missing is not None:
            payload["missing"] = self.missing
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "script" in data:
            args["script"] = data["script"]
        if "missing" in data:
            args["missing"] = data["missing"]        

        return cls(**args)


class ElasticsearchMaxSettings:
    script: typing.Optional['InlineScript']
    missing: typing.Optional[str]

    def __init__(self, script: typing.Optional['InlineScript'] = None, missing: typing.Optional[str] = None):
        self.script = script
        self.missing = missing

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.script is not None:
            payload["script"] = self.script
        if self.missing is not None:
            payload["missing"] = self.missing
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "script" in data:
            args["script"] = data["script"]
        if "missing" in data:
            args["missing"] = data["missing"]        

        return cls(**args)


class ElasticsearchSumSettings:
    script: typing.Optional['InlineScript']
    missing: typing.Optional[str]

    def __init__(self, script: typing.Optional['InlineScript'] = None, missing: typing.Optional[str] = None):
        self.script = script
        self.missing = missing

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.script is not None:
            payload["script"] = self.script
        if self.missing is not None:
            payload["missing"] = self.missing
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "script" in data:
            args["script"] = data["script"]
        if "missing" in data:
            args["missing"] = data["missing"]        

        return cls(**args)


class ElasticsearchAverageSettings:
    script: typing.Optional['InlineScript']
    missing: typing.Optional[str]

    def __init__(self, script: typing.Optional['InlineScript'] = None, missing: typing.Optional[str] = None):
        self.script = script
        self.missing = missing

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.script is not None:
            payload["script"] = self.script
        if self.missing is not None:
            payload["missing"] = self.missing
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "script" in data:
            args["script"] = data["script"]
        if "missing" in data:
            args["missing"] = data["missing"]        

        return cls(**args)


class ElasticsearchMovingFunctionSettings:
    window: typing.Optional[str]
    script: typing.Optional['InlineScript']
    shift: typing.Optional[str]

    def __init__(self, window: typing.Optional[str] = None, script: typing.Optional['InlineScript'] = None, shift: typing.Optional[str] = None):
        self.window = window
        self.script = script
        self.shift = shift

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.window is not None:
            payload["window"] = self.window
        if self.script is not None:
            payload["script"] = self.script
        if self.shift is not None:
            payload["shift"] = self.shift
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "window" in data:
            args["window"] = data["window"]
        if "script" in data:
            args["script"] = data["script"]
        if "shift" in data:
            args["shift"] = data["shift"]        

        return cls(**args)


class ElasticsearchLogsSettings:
    limit: typing.Optional[str]

    def __init__(self, limit: typing.Optional[str] = None):
        self.limit = limit

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.limit is not None:
            payload["limit"] = self.limit
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "limit" in data:
            args["limit"] = data["limit"]        

        return cls(**args)


class ElasticsearchRateSettings:
    unit: typing.Optional[str]
    mode: typing.Optional[str]

    def __init__(self, unit: typing.Optional[str] = None, mode: typing.Optional[str] = None):
        self.unit = unit
        self.mode = mode

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.unit is not None:
            payload["unit"] = self.unit
        if self.mode is not None:
            payload["mode"] = self.mode
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "unit" in data:
            args["unit"] = data["unit"]
        if "mode" in data:
            args["mode"] = data["mode"]        

        return cls(**args)


class ElasticsearchTopMetricsSettings:
    order: typing.Optional[str]
    order_by: typing.Optional[str]
    metrics: typing.Optional[list[str]]

    def __init__(self, order: typing.Optional[str] = None, order_by: typing.Optional[str] = None, metrics: typing.Optional[list[str]] = None):
        self.order = order
        self.order_by = order_by
        self.metrics = metrics

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.order is not None:
            payload["order"] = self.order
        if self.order_by is not None:
            payload["orderBy"] = self.order_by
        if self.metrics is not None:
            payload["metrics"] = self.metrics
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "order" in data:
            args["order"] = data["order"]
        if "orderBy" in data:
            args["order_by"] = data["orderBy"]
        if "metrics" in data:
            args["metrics"] = data["metrics"]        

        return cls(**args)


class ElasticsearchMetricAggregationWithMissingSupportSettings:
    missing: typing.Optional[str]

    def __init__(self, missing: typing.Optional[str] = None):
        self.missing = missing

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.missing is not None:
            payload["missing"] = self.missing
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "missing" in data:
            args["missing"] = data["missing"]        

        return cls(**args)


class ElasticsearchMetricAggregationWithInlineScriptSettings:
    script: typing.Optional['InlineScript']

    def __init__(self, script: typing.Optional['InlineScript'] = None):
        self.script = script

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.script is not None:
            payload["script"] = self.script
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "script" in data:
            args["script"] = data["script"]        

        return cls(**args)


class ElasticsearchMovingAverageEWMAModelSettingsSettings:
    alpha: typing.Optional[str]

    def __init__(self, alpha: typing.Optional[str] = None):
        self.alpha = alpha

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.alpha is not None:
            payload["alpha"] = self.alpha
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "alpha" in data:
            args["alpha"] = data["alpha"]        

        return cls(**args)


class ElasticsearchMovingAverageHoltModelSettingsSettings:
    alpha: typing.Optional[str]
    beta: typing.Optional[str]

    def __init__(self, alpha: typing.Optional[str] = None, beta: typing.Optional[str] = None):
        self.alpha = alpha
        self.beta = beta

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.alpha is not None:
            payload["alpha"] = self.alpha
        if self.beta is not None:
            payload["beta"] = self.beta
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "alpha" in data:
            args["alpha"] = data["alpha"]
        if "beta" in data:
            args["beta"] = data["beta"]        

        return cls(**args)


class ElasticsearchMovingAverageHoltWintersModelSettingsSettings:
    alpha: typing.Optional[str]
    beta: typing.Optional[str]
    gamma: typing.Optional[str]
    period: typing.Optional[str]
    pad: typing.Optional[bool]

    def __init__(self, alpha: typing.Optional[str] = None, beta: typing.Optional[str] = None, gamma: typing.Optional[str] = None, period: typing.Optional[str] = None, pad: typing.Optional[bool] = None):
        self.alpha = alpha
        self.beta = beta
        self.gamma = gamma
        self.period = period
        self.pad = pad

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.alpha is not None:
            payload["alpha"] = self.alpha
        if self.beta is not None:
            payload["beta"] = self.beta
        if self.gamma is not None:
            payload["gamma"] = self.gamma
        if self.period is not None:
            payload["period"] = self.period
        if self.pad is not None:
            payload["pad"] = self.pad
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "alpha" in data:
            args["alpha"] = data["alpha"]
        if "beta" in data:
            args["beta"] = data["beta"]
        if "gamma" in data:
            args["gamma"] = data["gamma"]
        if "period" in data:
            args["period"] = data["period"]
        if "pad" in data:
            args["pad"] = data["pad"]        

        return cls(**args)





def variant_config() -> cogruntime.DataqueryConfig:
    return cogruntime.DataqueryConfig(
        identifier="elasticsearch",
        from_json_hook=Dataquery.from_json,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import enum
from ..cog import variants as cogvariants
import typing
from ..cog import runtime as cogruntime


class QueryEditorMode(enum.StrEnum):
    CODE = "code"
    BUILDER = "builder"


class PromQueryFormat(enum.StrEnum):
    TIME_SERIES = "time_series"
    TABLE = "table"
    HEATMAP = "heatmap"


class Dataquery(cogvariants.Dataquery):
    # The actual expression/query that will be evaluated by Prometheus
    expr: str
    # Returns only the latest value that Prometheus has scraped for the requested time series
    instant: typing.Optional[bool]
    # Returns a Range vector, comprised of a set of time series containing a range of data points over time for each time series
    range_val: typing.Optional[bool]
    # Execute an additional query to identify interesting raw samples relevant for the given expr
    exemplar: typing.Optional[bool]
    # Specifies which editor is being used to prepare the query. It can be "code" or "builder"
    editor_mode: typing.Optional['QueryEditorMode']
    # Query format to determine how to display data points in panel. It can be "time_series", "table", "heatmap"
    format_val: typing.Optional['PromQueryFormat']
    # Series name override or template. Ex. {{hostname}} will be replaced with label value for hostname
    legend_format: typing.Optional[str]
    # @deprecated Used to specify how many times to divide max data points by. We use max data points under query options
    # See https://github.com/grafana/grafana/issues/48081
    interval_factor: typing.Optional[float]
    # A unique identifier for the query within the list of targets.
    # In server side expressions, the refId is used as a variable name to identify results.
    # By default, the UI will assign A->Z; however setting meaningful names may be useful.
    ref_id: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # Note this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Specify the query flavor
    # TODO make this required and give it a default
    query_type: typing.Optional[str]
    # For mixed data sources the selected datasource is on the query level.
    # For non mixed scenarios this is undefined.
    # TODO find a better way to do this ^ that's friendly to schema
    # TODO this shouldn't be unknown but DataSourceRef | null
    datasource: typing.Optional[object]
    # An additional lower limit for the step parameter of the Prometheus query and for the
    # `$__interval` and `$__rate_interval` variables.
    interval: typing.Optional[str]

    def __init__(self, expr: str = "", instant: typing.Optional[bool] = None, range_val: typing.Optional[bool] = None, exemplar: typing.Optional[bool] = None, editor_mode: typing.Optional['QueryEditorMode'] = None, format_val: typing.Optional['PromQueryFormat'] = None, legend_format: typing.Optional[str] = None, interval_factor: typing.Optional[float] = None, ref_id: str = "", hide: typing.Optional[bool] = None, query_type: typing.Optional[str] = None, datasource: typing.Optional[object] = None, interval: typing.Optional[str] = None):
        self.expr = expr
        self.instant = instant
        self.range_val = range_val
        self.exemplar = exemplar
        self.editor_mode = editor_mode
        self.format_val = format_val
        self.legend_format = legend_format
        self.interval_factor = interval_factor
        self.ref_id = ref_id
        self.hide = hide
        self.query_type = query_type
        self.datasource = datasource
        self.interval = interval

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "expr": self.expr,
            "refId": self.ref_id,
        }
        if self.instant is not None:
            payload["instant"] = self.instant
        if self.range_val is not None:
            payload["range"] = self.range_val
        if self.exemplar is not None:
            payload["exemplar"] = self.exemplar
        if self.editor_mode is not None:
            payload["editorMode"] = self.editor_mode
        if self.format_val is not None:
            payload["format"] = self.format_val
        if self.legend_format is not None:
            payload["legendFormat"] = self.legend_format
        if self.interval_factor is not None:
            payload["intervalFactor"] = self.interval_factor
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        if self.interval is not None:
            payload["interval"] = self.interval
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "expr" in data:
            args["expr"] = data["expr"]
        if "instant" in data:
            args["instant"] = data["instant"]
        if "range" in data:
            args["range_val"] = data["range"]
        if "exemplar" in data:
            args["exemplar"] = data["exemplar"]
        if "editorMode" in data:
            args["editor_mode"] = data["editorMode"]
        if "format" in data:
            args["format_val"] = data["format"]
        if "legendFormat" in data:
            args["legend_format"] = data["legendFormat"]
        if "intervalFactor" in data:
            args["interval_factor"] = data["intervalFactor"]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "datasource" in data:
            args["datasource"] = data["datasource"]
        if "interval" in data:
            args["interval"] = data["interval"]        

        return cls(**args)





def variant_config() -> cogruntime.DataqueryConfig:
    return cogruntime.DataqueryConfig(
        identifier="prometheus",
        from_json_hook=Dataquery.from_json,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import enum
import typing
from ..models import common
from ..cog import runtime as cogruntime


class HorizontalConstraint(enum.StrEnum):
    LEFT = "left"
    RIGHT = "right"
    LEFT_RIGHT = "leftright"
    CENTER = "center"
    SCALE = "scale"


class VerticalConstraint(enum.StrEnum):
    TOP = "top"
    BOTTOM = "bottom"
    TOP_BOTTOM = "topbottom"
    CENTER = "center"
    SCALE = "scale"


class Constraint:
    horizontal: typing.Optional['HorizontalConstraint']
    vertical: typing.Optional['VerticalConstraint']

    def __init__(self, horizontal: typing.Optional['HorizontalConstraint'] = None, vertical: typing.Optional['VerticalConstraint'] = None):
        self.horizontal = horizontal
        self.vertical = vertical

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.horizontal is not None:
            payload["horizontal"] = self.horizontal
        if self.vertical is not None:
            payload["vertical"] = self.vertical
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "horizontal" in data:
            args["horizontal"] = data["horizontal"]
        if "vertical" in data:
            args["vertical"] = data["vertical"]        

        return cls(**args)


class Placement:
    top: typing.Optional[float]
    left: typing.Optional[float]
    right: typing.Optional[float]
    bottom: typing.Optional[float]
    width: typing.Optional[float]
    height: typing.Optional[float]

    def __init__(self, top: typing.Optional[float] = None, left: typing.Optional[float] = None, right: typing.Optional[float] = None, bottom: typing.Optional[float] = None, width: typing.Optional[float] = None, height: typing.Optional[float] = None):
        self.top = top
        self.left = left
        self.right = right
        self.bottom = bottom
        self.width = width
        self.height = height

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.top is not None:
            payload["top"] = self.top
        if self.left is not None:
            payload["left"] = self.left
        if self.right is not None:
            payload["right"] = self.right
        if self.bottom is not None:
            payload["bottom"] = self.bottom
        if self.width is not None:
            payload["width"] = self.width
        if self.height is not None:
            payload["height"] = self.height
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "top" in data:
            args["top"] = data["top"]
        if "left" in data:
            args["left"] = data["left"]
        if "right" in data:
            args["right"] = data["right"]
        if "bottom" in data:
            args["bottom"] = data["bottom"]
        if "width" in data:
            args["width"] = data["width"]
        if "height" in data:
            args["height"] = data["height"]        

        return cls(**args)


class BackgroundImageSize(enum.StrEnum):
    ORIGINAL = "original"
    CONTAIN = "contain"
    COVER = "cover"
    FILL = "fill"
    TILE = "tile"


class BackgroundConfig:
    color: typing.Optional[common.ColorDimensionConfig]
    image: typing.Optional[common.ResourceDimensionConfig]
    size: typing.Optional['BackgroundImageSize']

    def __init__(self, color: typing.Optional[common.ColorDimensionConfig] = None, image: typing.Optional[common.ResourceDimensionConfig] = None, size: typing.Optional['BackgroundImageSize'] = None):
        self.color = color
        self.image = image
        self.size = size

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.color is not None:
            payload["color"] = self.color
        if self.image is not None:
            payload["image"] = self.image
        if self.size is not None:
            payload["size"] = self.size
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "color" in data:
            args["color"] = common.ColorDimensionConfig.from_json(data["color"])
        if "image" in data:
            args["image"] = common.ResourceDimensionConfig.from_json(data["image"])
        if "size" in data:
            args["size"] = data["size"]        

        return cls(**args)


class LineConfig:
    color: typing.Optional[common.ColorDimensionConfig]
    width: typing.Optional[float]

    def __init__(self, color: typing.Optional[common.ColorDimensionConfig] = None, width: typing.Optional[float] = None):
        self.color = color
        self.width = width

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.color is not None:
            payload["color"] = self.color
        if self.width is not None:
            payload["width"] = self.width
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "color" in data:
            args["color"] = common.ColorDimensionConfig.from_json(data["color"])
        if "width" in data:
            args["width"] = data["width"]        

        return cls(**args)


class HttpRequestMethod(enum.StrEnum):
    GET = "GET"
    POST = "POST"
    PUT = "PUT"


class ConnectionCoordinates:
    x: float
    y: float

    def __init__(self, x: float = 0, y: float = 0):
        self.x = x
        self.y = y

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "x": self.x,
            "y": self.y,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "x" in data:
            args["x"] = data["x"]
        if "y" in data:
            args["y"] = data["y"]        

        return cls(**args)


class ConnectionPath(enum.StrEnum):
    STRAIGHT = "straight"


class CanvasConnection:
    source: 'ConnectionCoordinates'
    target: 'ConnectionCoordinates'
    target_name: typing.Optional[str]
    path: 'ConnectionPath'
    color: typing.Optional[common.ColorDimensionConfig]
    size: typing.Optional[common.ScaleDimensionConfig]

    def __init__(self, source: typing.Optional['ConnectionCoordinates'] = None, target: typing.Optional['ConnectionCoordinates'] = None, target_name: typing.Optional[str] = None, path: typing.Optional['ConnectionPath'] = None, color: typing.Optional[common.ColorDimensionConfig] = None, size: typing.Optional[common.ScaleDimensionConfig] = None):
        self.source = source if source is not None else ConnectionCoordinates()
        self.target = target if target is not None else ConnectionCoordinates()
        self.target_name = target_name
        self.path = path if path is not None else ConnectionPath.STRAIGHT
        self.color = color
        self.size = size

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "source": self.source,
            "target": self.target,
            "path": self.path,
        }
        if self.target_name is not None:
            payload["targetName"] = self.target_name
        if self.color is not None:
            payload["color"] = self.color
        if self.size is not None:
            payload["size"] = self.size
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "source" in data:
            args["source"] = ConnectionCoordinates.from_json(data["source"])
        if "target" in data:
            args["target"] = ConnectionCoordinates.from_json(data["target"])
        if "targetName" in data:
            args["target_name"] = data["targetName"]
        if "path" in data:
            args["path"] = data["path"]
        if "color" in data:
            args["color"] = common.ColorDimensionConfig.from_json(data["color"])
        if "size" in data:
            args["size"] = common.ScaleDimensionConfig.from_json(data["size"])        

        return cls(**args)


class CanvasElementOptions:
    name: str
    type_val: str
    # TODO: figure out how to define this (element config(s))
    config: typing.Optional[object]
    constraint: typing.Optional['Constraint']
    placement: typing.Optional['Placement']
    background: typing.Optional['BackgroundConfig']
    border: typing.Optional['LineConfig']
    connections: typing.Optional[list['CanvasConnection']]

    def __init__(self, name: str = "", type_val: str = "", config: typing.Optional[object] = None, constraint: typing.Optional['Constraint'] = None, placement: typing.Optional['Placement'] = None, background: typing.Optional['BackgroundConfig'] = None, border: typing.Optional['LineConfig'] = None, connections: typing.Optional[list['CanvasConnection']] = None):
        self.name = name
        self.type_val = type_val
        self.config = config
        self.constraint = constraint
        self.placement = placement
        self.background = background
        self.border = border
        self.connections = connections

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "name": self.name,
            "type": self.type_val,
        }
        if self.config is not None:
            payload["config"] = self.config
        if self.constraint is not None:
            payload["constraint"] = self.constraint
        if self.placement is not None:
            payload["placement"] = self.placement
        if self.background is not None:
            payload["background"] = self.background
        if self.border is not None:
            payload["border"] = self.border
        if self.connections is not None:
            payload["connections"] = self.connections
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "name" in data:
            args["name"] = data["name"]
        if "type" in data:
            args["type_val"] = data["type"]
        if "config" in data:
            args["config"] = data["config"]
        if "constraint" in data:
            args["constraint"] = Constraint.from_json(data["constraint"])
        if "placement" in data:
            args["placement"] = Placement.from_json(data["placement"])
        if "background" in data:
            args["background"] = BackgroundConfig.from_json(data["background"])
        if "border" in data:
            args["border"] = LineConfig.from_json(data["border"])
        if "connections" in data:
            args["connections"] = [CanvasConnection.from_json(item) for item in data["connections"]]        

        return cls(**args)


class Options:
    # Enable inline editing
    inline_editing: bool
    # Show all available element types
    show_advanced_types: bool
    # The root element of canvas (frame), where all canvas elements are nested
    # TODO: Figure out how to define a default value for this
    root: 'CanvasOptionsRoot'

    def __init__(self, inline_editing: bool = True, show_advanced_types: bool = True, root: typing.Optional['CanvasOptionsRoot'] = None):
        self.inline_editing = inline_editing
        self.show_advanced_types = show_advanced_types
        self.root = root if root is not None else CanvasOptionsRoot()

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "inlineEditing": self.inline_editing,
            "showAdvancedTypes": self.show_advanced_types,
            "root": self.root,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "inlineEditing" in data:
            args["inline_editing"] = data["inlineEditing"]
        if "showAdvancedTypes" in data:
            args["show_advanced_types"] = data["showAdvancedTypes"]
        if "root" in data:
            args["root"] = CanvasOptionsRoot.from_json(data["root"])        

        return cls(**args)


class CanvasOptionsRoot:
    # Name of the root element
    name: str
    # Type of root element (frame)
    type_val: typing.Literal["frame"]
    # The list of canvas elements attached to the root element
    elements: list['CanvasElementOptions']

    def __init__(self, name: str = "", elements: typing.Optional[list['CanvasElementOptions']] = None):
        self.name = name
        self.type_val = "frame"
        self.elements = elements if elements is not None else []

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "name": self.name,
            "type": self.type_val,
            "elements": self.elements,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "name" in data:
            args["name"] = data["name"]
        if "elements" in data:
            args["elements"] = [CanvasElementOptions.from_json(item) for item in data["elements"]]        

        return cls(**args)





def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="canvas",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=None,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import alerting
from ..cog import variants as cogvariants


class Query(cogbuilder.Builder[alerting.Query]):
    _internal: alerting.Query

    def __init__(self, ref_id: str):
        self._internal = alerting.Query()        
        self._internal.ref_id = ref_id

    def build(self) -> alerting.Query:
        """
        Builds the object.
        """
        return self._internal    
    
    def datasource_uid(self, datasource_uid: str) -> typing.Self:    
        """
        Grafana data source unique identifier; it should be '__expr__' for a Server Side Expression operation.
        """
            
        self._internal.datasource_uid = datasource_uid
    
        return self
    
    def model(self, model: cogbuilder.Builder[cogvariants.Dataquery]) -> typing.Self:    
        """
        JSON is the raw JSON query and includes the above properties as well as custom properties.
        """
            
        model_resource = model.build()
        self._internal.model = model_resource
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        QueryType is an optional identifier for the type of query.
        It can be used to distinguish different types of queries.
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        RefID is the unique identifier of the query, set by the frontend call.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def relative_time_range(self, relative_time_range: alerting.RelativeTimeRange) -> typing.Self:    
        """
        RelativeTimeRange is the per query start and end time
        for requests.
        """
            
        self._internal.relative_time_range = relative_time_range
    
        return self
    


class RuleGroup(cogbuilder.Builder[alerting.RuleGroup]):
    _internal: alerting.RuleGroup

    def __init__(self, title: str):
        self._internal = alerting.RuleGroup()        
        self._internal.title = title

    def build(self) -> alerting.RuleGroup:
        """
        Builds the object.
        """
        return self._internal    
    
    def folder_uid(self, folder_uid: str) -> typing.Self:    
        self._internal.folder_uid = folder_uid
    
        return self
    
    def interval(self, interval: alerting.Duration) -> typing.Self:    
        """
        The interval, in seconds, at which all rules in the group are evaluated.
        If a group contains many rules, the rules are evaluated sequentially.
        """
            
        self._internal.interval = interval
    
        return self
    
    def rules(self, rules: list[cogbuilder.Builder[alerting.Rule]]) -> typing.Self:    
        rules_resources = [r1.build() for r1 in rules]
        self._internal.rules = rules_resources
    
        return self
    
    def with_rule(self, rule: cogbuilder.Builder[alerting.Rule]) -> typing.Self:    
        if self._internal.rules is None:
            self._internal.rules = []
        
        rule_resource = rule.build()
        self._internal.rules.append(rule_resource)
    
        return self
    
    def title(self, title: str) -> typing.Self:    
        self._internal.title = title
    
        return self
    


class ContactPoint(cogbuilder.Builder[alerting.ContactPoint]):    
    """
    EmbeddedContactPoint is the contact point type that is used
    by grafanas embedded alertmanager implementation.
    """
    
    _internal: alerting.ContactPoint

    def __init__(self):
        self._internal = alerting.ContactPoint()

    def build(self) -> alerting.ContactPoint:
        """
        Builds the object.
        """
        return self._internal    
    
    def disable_resolve_message(self, disable_resolve_message: bool) -> typing.Self:    
        self._internal.disable_resolve_message = disable_resolve_message
    
        return self
    
    def name(self, name: str) -> typing.Self:    
        """
        Name is used as grouping key in the UI. Contact points with the
        same name will be grouped in the UI.
        """
            
        self._internal.name = name
    
        return self
    
    def provenance(self, provenance: str) -> typing.Self:    
        self._internal.provenance = provenance
    
        return self
    
    def settings(self, settings: alerting.Json) -> typing.Self:    
        self._internal.settings = settings
    
        return self
    
    def type(self, type_val: typing.Literal["alertmanager", "dingding", "discord", "email", "googlechat", "kafka", "line", "opsgenie", "pagerduty", "pushover", "sensugo", "slack", "teams", "telegram", "threema", "victorops", "webhook", "wecom"]) -> typing.Self:    
        self._internal.type_val = type_val
    
        return self
    
    def uid(self, uid: str) -> typing.Self:    
        """
        UID is the unique identifier of the contact point. The UID can be
        set by the user.
        """
            
        if not len(uid) >= 1:
            raise ValueError("len(uid) must be >= 1")
        if not len(uid) <= 40:
            raise ValueError("len(uid) must be <= 40")
        self._internal.uid = uid
    
        return self
    


class Matcher(cogbuilder.Builder[alerting.Matcher]):
    _internal: alerting.Matcher

    def __init__(self):
        self._internal = alerting.Matcher()

    def build(self) -> alerting.Matcher:
        """
        Builds the object.
        """
        return self._internal    
    
    def name(self, name: str) -> typing.Self:    
        self._internal.name = name
    
        return self
    
    def type(self, type: alerting.MatchType) -> typing.Self:    
        self._internal.type = type
    
        return self
    
    def value(self, value: str) -> typing.Self:    
        self._internal.value = value
    
        return self
    


class MuteTiming(cogbuilder.Builder[alerting.MuteTiming]):
    _internal: alerting.MuteTiming

    def __init__(self):
        self._internal = alerting.MuteTiming()

    def build(self) -> alerting.MuteTiming:
        """
        Builds the object.
        """
        return self._internal    
    
    def name(self, name: str) -> typing.Self:    
        self._internal.name = name
    
        return self
    
    def time_intervals(self, time_intervals: list[cogbuilder.Builder[alerting.TimeInterval]]) -> typing.Self:    
        time_intervals_resources = [r1.build() for r1 in time_intervals]
        self._internal.time_intervals = time_intervals_resources
    
        return self
    


class NotificationTemplate(cogbuilder.Builder[alerting.NotificationTemplate]):
    _internal: alerting.NotificationTemplate

    def __init__(self):
        self._internal = alerting.NotificationTemplate()

    def build(self) -> alerting.NotificationTemplate:
        """
        Builds the object.
        """
        return self._internal    
    
    def name(self, name: str) -> typing.Self:    
        self._internal.name = name
    
        return self
    
    def provenance(self, provenance: alerting.Provenance) -> typing.Self:    
        self._internal.provenance = provenance
    
        return self
    
    def template(self, template: str) -> typing.Self:    
        self._internal.template = template
    
        return self
    


class Rule(cogbuilder.Builder[alerting.Rule]):
    _internal: alerting.Rule

    def __init__(self, title: str):
        self._internal = alerting.Rule()        
        if not len(title) >= 1:
            raise ValueError("len(title) must be >= 1")
        if not len(title) <= 190:
            raise ValueError("len(title) must be <= 190")
        self._internal.title = title

    def build(self) -> alerting.Rule:
        """
        Builds the object.
        """
        return self._internal    
    
    def annotations(self, annotations: dict[str, str]) -> typing.Self:    
        self._internal.annotations = annotations
    
        return self
    
    def condition(self, condition: str) -> typing.Self:    
        self._internal.condition = condition
    
        return self
    
    def queries(self, data: list[cogbuilder.Builder[alerting.Query]]) -> typing.Self:    
        data_resources = [r1.build() for r1 in data]
        self._internal.data = data_resources
    
        return self
    
    def with_query(self, data: cogbuilder.Builder[alerting.Query]) -> typing.Self:    
        if self._internal.data is None:
            self._internal.data = []
        
        data_resource = data.build()
        self._internal.data.append(data_resource)
    
        return self
    
    def exec_err_state(self, exec_err_state: typing.Literal["OK", "Alerting", "Error"]) -> typing.Self:    
        self._internal.exec_err_state = exec_err_state
    
        return self
    
    def folder_uid(self, folder_uid: str) -> typing.Self:    
        self._internal.folder_uid = folder_uid
    
        return self
    
    def for_val(self, for_val: str) -> typing.Self:    
        """
        The amount of time, in seconds, for which the rule must be breached for the rule to be considered to be Firing.
        Before this time has elapsed, the rule is only considered to be Pending.
        """
            
        self._internal.for_val = for_val
    
        return self
    
    def id(self, id_val: int) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def is_paused(self, is_paused: bool) -> typing.Self:    
        self._internal.is_paused = is_paused
    
        return self
    
    def labels(self, labels: dict[str, str]) -> typing.Self:    
        self._internal.labels = labels
    
        return self
    
    def no_data_state(self, no_data_state: typing.Literal["Alerting", "NoData", "OK"]) -> typing.Self:    
        self._internal.no_data_state = no_data_state
    
        return self
    
    def org_id(self, org_id: int) -> typing.Self:    
        self._internal.org_id = org_id
    
        return self
    
    def provenance(self, provenance: alerting.Provenance) -> typing.Self:    
        self._internal.provenance = provenance
    
        return self
    
    def rule_group(self, rule_group: str) -> typing.Self:    
        if not len(rule_group) >= 1:
            raise ValueError("len(rule_group) must be >= 1")
        if not len(rule_group) <= 190:
            raise ValueError("len(rule_group) must be <= 190")
        self._internal.rule_group = rule_group
    
        return self
    
    def title(self, title: str) -> typing.Self:    
        if not len(title) >= 1:
            raise ValueError("len(title) must be >= 1")
        if not len(title) <= 190:
            raise ValueError("len(title) must be <= 190")
        self._internal.title = title
    
        return self
    
    def uid(self, uid: str) -> typing.Self:    
        if not len(uid) >= 1:
            raise ValueError("len(uid) must be >= 1")
        if not len(uid) <= 40:
            raise ValueError("len(uid) must be <= 40")
        self._internal.uid = uid
    
        return self
    
    def updated(self, updated: str) -> typing.Self:    
        self._internal.updated = updated
    
        return self
    


class NotificationPolicy(cogbuilder.Builder[alerting.NotificationPolicy]):    
    """
    A Route is a node that contains definitions of how to handle alerts. This is modified
    from the upstream alertmanager in that it adds the ObjectMatchers property.
    """
    
    _internal: alerting.NotificationPolicy

    def __init__(self):
        self._internal = alerting.NotificationPolicy()

    def build(self) -> alerting.NotificationPolicy:
        """
        Builds the object.
        """
        return self._internal    
    
    def continue_val(self, continue_val: bool) -> typing.Self:    
        self._internal.continue_val = continue_val
    
        return self
    
    def group_by(self, group_by: list[str]) -> typing.Self:    
        self._internal.group_by = group_by
    
        return self
    
    def group_interval(self, group_interval: str) -> typing.Self:    
        self._internal.group_interval = group_interval
    
        return self
    
    def group_wait(self, group_wait: str) -> typing.Self:    
        self._internal.group_wait = group_wait
    
        return self
    
    def match(self, match: dict[str, str]) -> typing.Self:    
        """
        Deprecated. Remove before v1.0 release.
        """
            
        self._internal.match = match
    
        return self
    
    def match_re(self, match_re: alerting.MatchRegexps) -> typing.Self:    
        self._internal.match_re = match_re
    
        return self
    
    def matchers(self, matchers: alerting.Matchers) -> typing.Self:    
        """
        Matchers is a slice of Matchers that is sortable, implements Stringer, and
        provides a Matches method to match a LabelSet against all Matchers in the
        slice. Note that some users of Matchers might require it to be sorted.
        """
            
        self._internal.matchers = matchers
    
        return self
    
    def mute_time_intervals(self, mute_time_intervals: list[str]) -> typing.Self:    
        self._internal.mute_time_intervals = mute_time_intervals
    
        return self
    
    def object_matchers(self, object_matchers: alerting.ObjectMatchers) -> typing.Self:    
        """
        Matchers is a slice of Matchers that is sortable, implements Stringer, and
        provides a Matches method to match a LabelSet against all Matchers in the
        slice. Note that some users of Matchers might require it to be sorted.
        """
            
        self._internal.object_matchers = object_matchers
    
        return self
    
    def provenance(self, provenance: alerting.Provenance) -> typing.Self:    
        self._internal.provenance = provenance
    
        return self
    
    def receiver(self, receiver: str) -> typing.Self:    
        self._internal.receiver = receiver
    
        return self
    
    def repeat_interval(self, repeat_interval: str) -> typing.Self:    
        self._internal.repeat_interval = repeat_interval
    
        return self
    
    def routes(self, routes: list[cogbuilder.Builder[alerting.NotificationPolicy]]) -> typing.Self:    
        routes_resources = [r1.build() for r1 in routes]
        self._internal.routes = routes_resources
    
        return self
    


class TimeInterval(cogbuilder.Builder[alerting.TimeInterval]):
    _internal: alerting.TimeInterval

    def __init__(self):
        self._internal = alerting.TimeInterval()

    def build(self) -> alerting.TimeInterval:
        """
        Builds the object.
        """
        return self._internal    
    
    def times(self, times: list[cogbuilder.Builder[alerting.TimeRange]]) -> typing.Self:    
        times_resources = [r1.build() for r1 in times]
        self._internal.times = times_resources
    
        return self
    
    def weekdays(self, weekdays: list[cogbuilder.Builder[alerting.WeekdayRange]]) -> typing.Self:    
        weekdays_resources = [r1.build() for r1 in weekdays]
        self._internal.weekdays = weekdays_resources
    
        return self
    
    def days_of_month(self, days_of_month: list[cogbuilder.Builder[alerting.DayOfMonthRange]]) -> typing.Self:    
        days_of_month_resources = [r1.build() for r1 in days_of_month]
        self._internal.days_of_month = days_of_month_resources
    
        return self
    
    def months(self, months: list[cogbuilder.Builder[alerting.MonthRange]]) -> typing.Self:    
        months_resources = [r1.build() for r1 in months]
        self._internal.months = months_resources
    
        return self
    
    def years(self, years: list[cogbuilder.Builder[alerting.YearRange]]) -> typing.Self:    
        years_resources = [r1.build() for r1 in years]
        self._internal.years = years_resources
    
        return self
    
    def location(self, location: alerting.Location) -> typing.Self:    
        self._internal.location = location
    
        return self
    


class TimeRange(cogbuilder.Builder[alerting.TimeRange]):    
    """
    Redefining this to avoid an import cycle
    """
    
    _internal: alerting.TimeRange

    def __init__(self):
        self._internal = alerting.TimeRange()

    def build(self) -> alerting.TimeRange:
        """
        Builds the object.
        """
        return self._internal    
    
    def from_val(self, from_val: str) -> typing.Self:    
        self._internal.from_val = from_val
    
        return self
    
    def to(self, to: str) -> typing.Self:    
        self._internal.to = to
    
        return self
    


class WeekdayRange(cogbuilder.Builder[alerting.WeekdayRange]):
    _internal: alerting.WeekdayRange

    def __init__(self):
        self._internal = alerting.WeekdayRange()

    def build(self) -> alerting.WeekdayRange:
        """
        Builds the object.
        """
        return self._internal    
    
    def begin(self, begin: int) -> typing.Self:    
        self._internal.begin = begin
    
        return self
    
    def end(self, end: int) -> typing.Self:    
        self._internal.end = end
    
        return self
    


class DayOfMonthRange(cogbuilder.Builder[alerting.DayOfMonthRange]):
    _internal: alerting.DayOfMonthRange

    def __init__(self):
        self._internal = alerting.DayOfMonthRange()

    def build(self) -> alerting.DayOfMonthRange:
        """
        Builds the object.
        """
        return self._internal    
    
    def begin(self, begin: int) -> typing.Self:    
        self._internal.begin = begin
    
        return self
    
    def end(self, end: int) -> typing.Self:    
        self._internal.end = end
    
        return self
    


class YearRange(cogbuilder.Builder[alerting.YearRange]):
    _internal: alerting.YearRange

    def __init__(self):
        self._internal = alerting.YearRange()

    def build(self) -> alerting.YearRange:
        """
        Builds the object.
        """
        return self._internal    
    
    def begin(self, begin: int) -> typing.Self:    
        self._internal.begin = begin
    
        return self
    
    def end(self, end: int) -> typing.Self:    
        self._internal.end = end
    
        return self
    


class MonthRange(cogbuilder.Builder[alerting.MonthRange]):
    _internal: alerting.MonthRange

    def __init__(self):
        self._internal = alerting.MonthRange()

    def build(self) -> alerting.MonthRange:
        """
        Builds the object.
        """
        return self._internal    
    
    def begin(self, begin: int) -> typing.Self:    
        self._internal.begin = begin
    
        return self
    
    def end(self, end: int) -> typing.Self:    
        self._internal.end = end
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import runtime as cogruntime


class ArcOption:
    # Field from which to get the value. Values should be less than 1, representing fraction of a circle.
    field: typing.Optional[str]
    # The color of the arc.
    color: typing.Optional[str]

    def __init__(self, field: typing.Optional[str] = None, color: typing.Optional[str] = None):
        self.field = field
        self.color = color

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.field is not None:
            payload["field"] = self.field
        if self.color is not None:
            payload["color"] = self.color
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "field" in data:
            args["field"] = data["field"]
        if "color" in data:
            args["color"] = data["color"]        

        return cls(**args)


class NodeOptions:
    # Unit for the main stat to override what ever is set in the data frame.
    main_stat_unit: typing.Optional[str]
    # Unit for the secondary stat to override what ever is set in the data frame.
    secondary_stat_unit: typing.Optional[str]
    # Define which fields are shown as part of the node arc (colored circle around the node).
    arcs: typing.Optional[list['ArcOption']]

    def __init__(self, main_stat_unit: typing.Optional[str] = None, secondary_stat_unit: typing.Optional[str] = None, arcs: typing.Optional[list['ArcOption']] = None):
        self.main_stat_unit = main_stat_unit
        self.secondary_stat_unit = secondary_stat_unit
        self.arcs = arcs

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.main_stat_unit is not None:
            payload["mainStatUnit"] = self.main_stat_unit
        if self.secondary_stat_unit is not None:
            payload["secondaryStatUnit"] = self.secondary_stat_unit
        if self.arcs is not None:
            payload["arcs"] = self.arcs
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "mainStatUnit" in data:
            args["main_stat_unit"] = data["mainStatUnit"]
        if "secondaryStatUnit" in data:
            args["secondary_stat_unit"] = data["secondaryStatUnit"]
        if "arcs" in data:
            args["arcs"] = [ArcOption.from_json(item) for item in data["arcs"]]        

        return cls(**args)


class EdgeOptions:
    # Unit for the main stat to override what ever is set in the data frame.
    main_stat_unit: typing.Optional[str]
    # Unit for the secondary stat to override what ever is set in the data frame.
    secondary_stat_unit: typing.Optional[str]

    def __init__(self, main_stat_unit: typing.Optional[str] = None, secondary_stat_unit: typing.Optional[str] = None):
        self.main_stat_unit = main_stat_unit
        self.secondary_stat_unit = secondary_stat_unit

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.main_stat_unit is not None:
            payload["mainStatUnit"] = self.main_stat_unit
        if self.secondary_stat_unit is not None:
            payload["secondaryStatUnit"] = self.secondary_stat_unit
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "mainStatUnit" in data:
            args["main_stat_unit"] = data["mainStatUnit"]
        if "secondaryStatUnit" in data:
            args["secondary_stat_unit"] = data["secondaryStatUnit"]        

        return cls(**args)


class Options:
    nodes: typing.Optional['NodeOptions']
    edges: typing.Optional['EdgeOptions']

    def __init__(self, nodes: typing.Optional['NodeOptions'] = None, edges: typing.Optional['EdgeOptions'] = None):
        self.nodes = nodes
        self.edges = edges

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.nodes is not None:
            payload["nodes"] = self.nodes
        if self.edges is not None:
            payload["edges"] = self.edges
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "nodes" in data:
            args["nodes"] = NodeOptions.from_json(data["nodes"])
        if "edges" in data:
            args["edges"] = EdgeOptions.from_json(data["edges"])        

        return cls(**args)





def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="nodeGraph",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=None,
    )

